@@ -62,12 +62,12 @@ static int PARI_SIGINT_block = 0;
 static int PARI_SIGINT_pending = 0;
 #define paricfg_version NULL
 #endif
-#include "struct_signals.h"
-
-/* Some systems have MAP_ANON but not MAP_ANONYMOUS */
-#ifndef MAP_ANONYMOUS
-#define MAP_ANONYMOUS MAP_ANON
+#if HAVE_WINDOWS_H
+/* We must include <windows.h> after <pari.h>
+ * See https://github.com/sagemath/cysignals/issues/107 */
+#include <windows.h>
 #endif
+#include "struct_signals.h"
 
 
 #if ENABLE_DEBUG_CYSIGNALS
@@ -91,6 +91,9 @@ static sigset_t sigmask_with_sigint;
 static cyjmp_buf trampoline_setup;
 static sigjmp_buf trampoline;
 
+static void setup_cysignals_handlers(void);
+static void cysigs_interrupt_handler(int sig);
+static void cysigs_signal_handler(int sig);
 
 static void do_raise_exception(int sig);
 static void sigdie(int sig, const char* s);
@@ -146,7 +149,9 @@ static inline void sigdie_for_sig(int sig, int inside)
 {
     sig_reset_defaults();
 
-    /* Quit Python with an appropriate message. */
+    /* Quit Python with an appropriate message.
+       Make sure to check the standard signals from the C standard first,
+       in case systems alias some of these constants. */
     if (inside)
         sigdie(sig, "An error occurred during signal handling.");
     else if (sig == SIGILL)
@@ -227,7 +232,7 @@ static void cysigs_interrupt_handler(int sig)
  * Outside of sig_on(), we terminate Python. */
 static void cysigs_signal_handler(int sig)
 {
-    sig_atomic_t inside = cysigs.inside_signal_handler;
+    int inside = cysigs.inside_signal_handler;
     cysigs.inside_signal_handler = 1;
 
     if (inside == 0 && cysigs.sig_on_count > 0 && sig != SIGQUIT)
@@ -312,14 +317,14 @@ static void setup_trampoline(void)
     pthread_t child;
     pthread_attr_t attr;
     void* trampolinestack;
-    size_t trampolinestacksize = 1 << 16;
+    size_t trampolinestacksize = 1 << 17;
 
 #ifdef PTHREAD_STACK_MIN
     if (trampolinestacksize < PTHREAD_STACK_MIN)
         trampolinestacksize = PTHREAD_STACK_MIN;
 #endif
     trampolinestack = malloc(trampolinestacksize + 4096);
-    if (!trampolinestack) {perror("malloc"); exit(1);}
+    if (!trampolinestack) {perror("cysignals malloc"); exit(1);}
 
     /* Align trampolinestack on a multiple of 4096 bytes.
      * This seems to be needed in particular on OS X. */
@@ -328,14 +333,14 @@ static void setup_trampoline(void)
     trampolinestack = (void*)addr;
 
     ret = pthread_attr_init(&attr);
-    if (ret) {errno = ret; perror("pthread_attr_init"); exit(1);}
+    if (ret) {errno = ret; perror("cysignals pthread_attr_init"); exit(1);}
     ret = pthread_attr_setstack(&attr, trampolinestack, trampolinestacksize);
-    if (ret) {errno = ret; perror("pthread_attr_setstack"); exit(1);}
+    if (ret) {errno = ret; perror("cysignals pthread_attr_setstack"); exit(1);}
     ret = pthread_create(&child, &attr, _sig_on_trampoline, NULL);
-    if (ret) {errno = ret; perror("pthread_create"); exit(1);}
+    if (ret) {errno = ret; perror("cysignals pthread_create"); exit(1);}
     pthread_attr_destroy(&attr);
     ret = pthread_join(child, NULL);
-    if (ret) {errno = ret; perror("pthread_join"); exit(1);}
+    if (ret) {errno = ret; perror("cysignals pthread_join"); exit(1);}
 
     if (cysetjmp(cysigs.env) == 0)
     {
@@ -420,6 +425,7 @@ static void _sig_off_warning(const char* file, int line)
 
 static void setup_alt_stack(void)
 {
+#if HAVE_SIGALTSTACK
     /* Static space for the alternate signal stack. The size should be
      * of the form MINSIGSTKSZ + constant. The constant is chosen rather
      * ad hoc but sufficiently large. */
@@ -429,7 +435,8 @@ static void setup_alt_stack(void)
     ss.ss_sp = alt_stack;
     ss.ss_size = sizeof(alt_stack);
     ss.ss_flags = 0;
-    if (sigaltstack(&ss, NULL) == -1) {perror("sigaltstack"); exit(1);}
+    if (sigaltstack(&ss, NULL) == -1) {perror("cysignals sigaltstack"); exit(1);}
+#endif
 #if defined(__CYGWIN__) && defined(__x86_64__)
     cygwin_setup_alt_stack();
 #endif
@@ -465,21 +472,21 @@ static void setup_cysignals_handlers(void)
     /* Handlers for interrupt-like signals */
     sa.sa_handler = cysigs_interrupt_handler;
     sa.sa_flags = 0;
-    if (sigaction(SIGHUP, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGINT, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGALRM, &sa, NULL)) {perror("sigaction"); exit(1);}
+    if (sigaction(SIGHUP, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGINT, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGALRM, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
 
     /* Handlers for critical signals */
     sa.sa_handler = cysigs_signal_handler;
     /* Allow signals during signal handling, we have code to deal with
      * this case. */
     sa.sa_flags = SA_NODEFER | SA_ONSTACK;
-    if (sigaction(SIGQUIT, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGILL, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGABRT, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGFPE, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGBUS, &sa, NULL)) {perror("sigaction"); exit(1);}
-    if (sigaction(SIGSEGV, &sa, NULL)) {perror("sigaction"); exit(1);}
+    if (sigaction(SIGQUIT, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGILL, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGABRT, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGFPE, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGBUS, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
+    if (sigaction(SIGSEGV, &sa, NULL)) {perror("cysignals sigaction"); exit(1);}
 }
 
 
@@ -518,13 +525,15 @@ static void print_enhanced_backtrace(void)
     fflush(stdout);
     fflush(stderr);
 
+    /* Enhanced backtraces are only supported on POSIX systems */
+#if HAVE_FORK
     pid_t parent_pid = getpid();
     pid_t pid = fork();
 
     if (pid < 0)
     {
         /* Failed to fork: no problem, just ignore */
-        perror("fork");
+        perror("cysignals fork");
         return;
     }
 
@@ -547,11 +556,12 @@ static void print_enhanced_backtrace(void)
         argv[3] = pid_str;
         argv[4] = NULL;
         execvp(path, argv);
-        perror("Failed to execute cysignals-CSI");
+        perror("cysignals failed to execute cysignals-CSI");
         exit(2);
     }
     /* Wait for cysignals-CSI to finish */
     waitpid(pid, NULL, 0);
+#endif
 
     print_sep();
 }
@@ -590,10 +600,8 @@ static void sigdie(int sig, const char* s)
     }
 
 dienow:
-    /* Suicide with signal ``sig``. We intentionally use kill(getpid())
-     * instead of raise() because we want to kill all threads in a
-     * multi-threaded program. */
-    kill(getpid(), sig);
+    /* Suicide with signal ``sig``. */
+    raise(sig);
 
     /* We should be dead! */
     exit(128 + sig);
